@@ -24,15 +24,6 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-using Gtk;
-using System;
-using MonoDevelop.Database.Sql;
-using MonoDevelop.Core;
-using MonoDevelop.Core.Gui;
-using MonoDevelop.Ide.Gui;
-using MonoDevelop.Ide.Gui.Pads;
-using MonoDevelop.Components.Commands;
-using MonoDevelop.Database.Components;
 
 namespace MonoDevelop.Database.ConnectionManager
 {
