@@ -36,7 +36,19 @@ namespace MonoDevelop.Database.Sql.Npgsql
 	{
 		public bool ShowCreateDatabaseDialog (IDbFactory factory)
 		{
-			return false;
+			NpgsqlCreateDatabaseDialog dialog = new NpgsqlCreateDatabaseDialog (factory);
+			int resp;
+			do {
+				resp = dialog.Run ();
+			} while (resp != (int)ResponseType.Cancel && 
+				    	     resp != (int)ResponseType.Ok && 
+				    		resp != (int)ResponseType.DeleteEvent);
+			dialog.Destroy ();
+			if (resp == (int)ResponseType.Ok)
+				return true;
+			else
+				return false;
+
 		}
 
 		public bool ShowAddConnectionDialog (IDbFactory factory)
@@ -45,10 +57,17 @@ namespace MonoDevelop.Database.Sql.Npgsql
 			return RunDialog (dlg);
 		}
 		
-		public bool ShowEditConnectionDialog (IDbFactory factory, DatabaseConnectionSettings settings)
+		public bool ShowEditConnectionDialog (IDbFactory factory, 
+		                                      DatabaseConnectionSettings settings, 
+		                                      out DatabaseConnectionSettings newSettings)
 		{
 			DatabaseConnectionSettingsDialog dlg = new DatabaseConnectionSettingsDialog (factory, settings);
-			return RunDialog (dlg);
+			bool result = RunDialog (dlg);
+			if (result)
+				newSettings = dlg.ConnectionSettings;
+			else
+				newSettings = null;			
+			return result;
 		}
 
 		public bool ShowTableEditorDialog (IEditSchemaProvider schemaProvider, TableSchema table, bool create)
@@ -86,9 +105,19 @@ namespace MonoDevelop.Database.Sql.Npgsql
 		private bool RunDialog (Dialog dlg)
 		{
 			bool result = false;
+			// If the Preview Dialog is canceled, don't execute and don't close the Editor Dialog.
 			try {	
-				if (dlg.Run () == (int)ResponseType.Ok)
+				int resp;
+					do {
+						resp = dlg.Run ();
+				    } while (resp != (int)ResponseType.Cancel && 
+				    	     resp != (int)ResponseType.Ok && 
+				    		resp != (int)ResponseType.DeleteEvent);
+					
+				if (resp == (int)ResponseType.Ok)
 					result = true;
+				else
+					result = false;
 			} finally {
 				dlg.Destroy ();
 			}
