@@ -1646,5 +1646,11 @@ namespace MonoDevelop.Database.Sql.Npgsql
 				}
 			}
 		}
+		
+		public override string GetMimeType ()
+		{
+			return "text/x-npgsql";
+		}
+
 	}
 }
