@@ -36,7 +36,18 @@ namespace MonoDevelop.Database.Sql.SqlServer
 	{
 		public bool ShowCreateDatabaseDialog (IDbFactory factory)
 		{
-			return false;
+			SqlServerCreateDatabaseDialog dialog = new SqlServerCreateDatabaseDialog (factory);
+			int resp;
+			do {
+				resp = dialog.Run ();
+			} while (resp != (int)ResponseType.Cancel && 
+				    	     resp != (int)ResponseType.Ok && 
+				    		resp != (int)ResponseType.DeleteEvent);
+			dialog.Destroy ();
+			if (resp == (int)ResponseType.Ok)
+				return true;
+			else
+				return false;
 		}
 
 		public bool ShowAddConnectionDialog (IDbFactory factory)
@@ -49,12 +60,13 @@ namespace MonoDevelop.Database.Sql.SqlServer
 		                                      DatabaseConnectionSettings settings, 
 		                                      out DatabaseConnectionSettings newSettings)
 		{
+			
 			DatabaseConnectionSettingsDialog dlg = new DatabaseConnectionSettingsDialog (factory, settings);
 			bool result = RunDialog (dlg);
 			if (result)
 				newSettings = dlg.ConnectionSettings;
 			else
-				newSettings = null;	
+				newSettings = null;
 			return result;
 		}
 
